@@ -18,21 +18,15 @@ module GraphQL
         include_built_in_directives: false, include_built_in_scalars: false, always_include_schema: false
       )
         @schema = schema
+        @context = context
         @always_include_schema = always_include_schema
         @include_introspection_types = include_introspection_types
         @include_built_in_scalars = include_built_in_scalars
         @include_built_in_directives = include_built_in_directives
         @include_one_of = false
 
-        schema_context = schema.context_class.new(query: nil, object: nil, schema: schema, values: context)
-
-
-        @warden = @schema.warden_class.new(
-          schema: @schema,
-          context: schema_context,
-        )
-
-        schema_context.warden = @warden
+        dummy_query = @schema.query_class.new(@schema, "{ __typename }", validate: false, context: context)
+        @types = dummy_query.types # rubocop:disable Development/ContextIsPassedCop
       end
 
       def document
@@ -44,9 +38,9 @@ module GraphQL
       def build_schema_node
         if !schema_respects_root_name_conventions?(@schema)
           GraphQL::Language::Nodes::SchemaDefinition.new(
-            query: (q = warden.root_type_for_operation("query")) && q.graphql_name,
-            mutation: (m = warden.root_type_for_operation("mutation")) && m.graphql_name,
-            subscription: (s = warden.root_type_for_operation("subscription")) && s.graphql_name,
+            query:  @types.query_root&.graphql_name,
+            mutation: @types.mutation_root&.graphql_name,
+            subscription: @types.subscription_root&.graphql_name,
             directives: definition_directives(@schema, :schema_directives)
           )
         else
@@ -57,16 +51,18 @@ module GraphQL
       end
 
       def build_object_type_node(object_type)
-        ints = warden.interfaces(object_type)
-        if ints.any?
-          ints.sort_by!(&:graphql_name)
+        ints = @types.interfaces(object_type)
+
+        if !ints.empty?
+          ints = ints.sort_by(&:graphql_name)
           ints.map! { |iface| build_type_name_node(iface) }
         end
 
         GraphQL::Language::Nodes::ObjectTypeDefinition.new(
           name: object_type.graphql_name,
+          comment: object_type.comment,
           interfaces: ints,
-          fields: build_field_nodes(warden.fields(object_type)),
+          fields: build_field_nodes(@types.fields(object_type)),
           description: object_type.description,
           directives: directives(object_type),
         )
@@ -75,7 +71,8 @@ module GraphQL
       def build_field_node(field)
         GraphQL::Language::Nodes::FieldDefinition.new(
           name: field.graphql_name,
-          arguments: build_argument_nodes(warden.arguments(field)),
+          comment: field.comment,
+          arguments: build_argument_nodes(@types.arguments(field)),
           type: build_type_name_node(field.type),
           description: field.description,
           directives: directives(field),
@@ -85,8 +82,9 @@ module GraphQL
       def build_union_type_node(union_type)
         GraphQL::Language::Nodes::UnionTypeDefinition.new(
           name: union_type.graphql_name,
+          comment: union_type.comment,
           description: union_type.description,
-          types: warden.possible_types(union_type).sort_by(&:graphql_name).map { |type| build_type_name_node(type) },
+          types: @types.possible_types(union_type).sort_by(&:graphql_name).map { |type| build_type_name_node(type) },
           directives: directives(union_type),
         )
       end
@@ -94,9 +92,10 @@ module GraphQL
       def build_interface_type_node(interface_type)
         GraphQL::Language::Nodes::InterfaceTypeDefinition.new(
           name: interface_type.graphql_name,
-          interfaces: warden.interfaces(interface_type).sort_by(&:graphql_name).map { |type| build_type_name_node(type) },
+          comment: interface_type.comment,
+          interfaces: @types.interfaces(interface_type).sort_by(&:graphql_name).map { |type| build_type_name_node(type) },
           description: interface_type.description,
-          fields: build_field_nodes(warden.fields(interface_type)),
+          fields: build_field_nodes(@types.fields(interface_type)),
           directives: directives(interface_type),
         )
       end
@@ -104,7 +103,8 @@ module GraphQL
       def build_enum_type_node(enum_type)
         GraphQL::Language::Nodes::EnumTypeDefinition.new(
           name: enum_type.graphql_name,
-          values: warden.enum_values(enum_type).sort_by(&:graphql_name).map do |enum_value|
+          comment: enum_type.comment,
+          values: @types.enum_values(enum_type).sort_by(&:graphql_name).map do |enum_value|
             build_enum_value_node(enum_value)
           end,
           description: enum_type.description,
@@ -115,6 +115,7 @@ module GraphQL
       def build_enum_value_node(enum_value)
         GraphQL::Language::Nodes::EnumValueDefinition.new(
           name: enum_value.graphql_name,
+          comment: enum_value.comment,
           description: enum_value.description,
           directives: directives(enum_value),
         )
@@ -123,6 +124,7 @@ module GraphQL
       def build_scalar_type_node(scalar_type)
         GraphQL::Language::Nodes::ScalarTypeDefinition.new(
           name: scalar_type.graphql_name,
+          comment: scalar_type.comment,
           description: scalar_type.description,
           directives: directives(scalar_type),
         )
@@ -137,6 +139,7 @@ module GraphQL
 
         argument_node = GraphQL::Language::Nodes::InputValueDefinition.new(
           name: argument.graphql_name,
+          comment: argument.comment,
           description: argument.description,
           type: build_type_name_node(argument.type),
           default_value: default_value,
@@ -149,7 +152,8 @@ module GraphQL
       def build_input_object_node(input_object)
         GraphQL::Language::Nodes::InputObjectTypeDefinition.new(
           name: input_object.graphql_name,
-          fields: build_argument_nodes(warden.arguments(input_object)),
+          comment: input_object.comment,
+          fields: build_argument_nodes(@types.arguments(input_object)),
           description: input_object.description,
           directives: directives(input_object),
         )
@@ -159,7 +163,7 @@ module GraphQL
         GraphQL::Language::Nodes::DirectiveDefinition.new(
           name: directive.graphql_name,
           repeatable: directive.repeatable?,
-          arguments: build_argument_nodes(warden.arguments(directive)),
+          arguments: build_argument_nodes(@types.arguments(directive)),
           locations: build_directive_location_nodes(directive.locations),
           description: directive.description,
         )
@@ -204,7 +208,7 @@ module GraphQL
         when "INPUT_OBJECT"
           GraphQL::Language::Nodes::InputObject.new(
             arguments: default_value.to_h.map do |arg_name, arg_value|
-              args = @warden.arguments(type)
+              args = @types.arguments(type)
               arg = args.find { |a| a.keyword.to_s == arg_name.to_s }
               if arg.nil?
                 raise ArgumentError, "No argument definition on #{type.graphql_name} for argument: #{arg_name.inspect} (expected one of: #{args.map(&:keyword)})"
@@ -244,7 +248,7 @@ module GraphQL
       end
 
       def build_argument_nodes(arguments)
-        if arguments.any?
+        if !arguments.empty?
           nodes = arguments.map { |arg| build_argument_node(arg) }
           nodes.sort_by!(&:name)
           nodes
@@ -260,13 +264,39 @@ module GraphQL
       end
 
       def build_definition_nodes
-        dirs_to_build = warden.directives
+        dirs_to_build = @types.directives
         if !include_built_in_directives
           dirs_to_build = dirs_to_build.reject { |directive| directive.default_directive? }
         end
         definitions = build_directive_nodes(dirs_to_build)
+        all_types = @types.all_types
+        type_nodes = build_type_definition_nodes(all_types)
+
+        if !(ex_t = schema.extra_types).empty?
+          dummy_query = Class.new(GraphQL::Schema::Object) do
+            graphql_name "DummyQuery"
+            (all_types + ex_t).each_with_index do |type, idx|
+              if !type.kind.input_object? && !type.introspection?
+                field "f#{idx}", type
+              end
+            end
+          end
+
+          extra_types_schema = Class.new(GraphQL::Schema) do
+            query(dummy_query)
+          end
+
+          extra_types_types = GraphQL::Query.new(extra_types_schema, "{ __typename }", context: @context).types  # rubocop:disable Development/ContextIsPassedCop
+          # Temporarily replace `@types` with something from this example schema.
+          # It'd be much nicer to pass this in, but that would be a big refactor :S
+          prev_types = @types
+          @types = extra_types_types
+          type_nodes += build_type_definition_nodes(ex_t)
+          @types = prev_types
+        end
+
+        type_nodes.sort_by!(&:name)
 
-        type_nodes = build_type_definition_nodes(warden.reachable_types + schema.extra_types)
         if @include_one_of
           # This may have been set to true when iterating over all types
           definitions.concat(build_directive_nodes([GraphQL::Schema::Directive::OneOf]))
@@ -289,9 +319,7 @@ module GraphQL
           types = types.reject { |type| type.kind.scalar? && type.default_scalar? }
         end
 
-        types
-          .map { |type| build_type_definition_node(type) }
-          .sort_by(&:name)
+        types.map { |type| build_type_definition_node(type) }
       end
 
       def build_field_nodes(fields)
@@ -319,10 +347,11 @@ module GraphQL
       end
 
       def definition_directives(member, directives_method)
-        dirs = if !member.respond_to?(directives_method) || member.directives.empty?
+        if !member.respond_to?(directives_method) || member.directives.empty?
           EmptyObjects::EMPTY_ARRAY
         else
-          member.public_send(directives_method).map do |dir|
+          visible_directives = member.public_send(directives_method).select { |dir| @types.directive_exists?(dir.graphql_name) }
+          visible_directives.map! do |dir|
             args = []
             dir.arguments.argument_values.each_value do |arg_value| # rubocop:disable Development/ContextIsPassedCop -- directive instance method
               arg_defn = arg_value.definition
@@ -346,12 +375,12 @@ module GraphQL
               arguments: args
             )
           end
-        end
 
-        dirs
+          visible_directives
+        end
       end
 
-      attr_reader :schema, :warden, :always_include_schema,
+      attr_reader :schema, :always_include_schema,
         :include_introspection_types, :include_built_in_directives, :include_built_in_scalars
     end
   end
