@@ -63,6 +63,10 @@ describe "GraphQL::Authorization" do
       def visible?(context)
         super && (context[:hide] ? @role != :hidden : true)
       end
+
+      def authorized?(context)
+        super && (context[:authorized] ? true : @role != :unauthorized)
+      end
     end
 
     class BaseEnum < GraphQL::Schema::Enum
@@ -72,18 +76,25 @@ describe "GraphQL::Authorization" do
     module HiddenInterface
       include BaseInterface
 
-      def self.visible?(ctx)
-        super && !ctx[:hide]
-      end
+      definition_methods do
+        def visible?(ctx)
+          super && !ctx[:hide]
+        end
 
-      def self.resolve_type(obj, ctx)
-        HiddenObject
+        def resolve_type(obj, ctx)
+          HiddenObject
+        end
       end
     end
 
     module HiddenDefaultInterface
-      include BaseInterface
-      # visible? will call the super method
+      if GraphQL::Schema.use_visibility_profile?
+        include HiddenInterface
+      else
+        # Warden will detect no possible types
+        include BaseInterface
+      end
+
       def self.resolve_type(obj, ctx)
         HiddenObject
       end
@@ -339,7 +350,7 @@ describe "GraphQL::Authorization" do
       query(Query)
       mutation(Mutation)
       directive(Nothing)
-
+      use GraphQL::Schema::Warden if ADD_WARDEN
       lazy_resolve(Box, :value)
 
       def self.unauthorized_object(err)
@@ -353,13 +364,11 @@ describe "GraphQL::Authorization" do
           raise GraphQL::ExecutionError, "Unauthorized #{err.type.graphql_name}: #{err.object.inspect}"
         end
       end
-
-      # use GraphQL::Backtrace
     end
 
     class SchemaWithFieldHook < GraphQL::Schema
       query(Query)
-
+      use GraphQL::Schema::Warden if ADD_WARDEN
       lazy_resolve(Box, :value)
 
       def self.unauthorized_field(err)
@@ -393,7 +402,7 @@ describe "GraphQL::Authorization" do
 
       error_queries.each do |name, q|
         hidden_res = auth_execute(q, context: { hide: true})
-        assert_equal ["Field '#{name}' doesn't exist on type 'Query'"], hidden_res["errors"].map { |e| e["message"] }
+        assert_equal ["Field '#{name}' doesn't exist on type 'Query'#{name == "hiddenDefaultInterface" ?  "" : " (Did you mean `hiddenConnection`?)"}"], hidden_res["errors"].map { |e| e["message"] }
 
         visible_res = auth_execute(q)
         # Both fields exist; the interface resolves to the object type, though
@@ -453,7 +462,7 @@ describe "GraphQL::Authorization" do
       assert_equal "RelayObjectEdge", visible_res["data"]["hiddenEdge"]["__typename"]
     end
 
-    it "treats hidden enum values as non-existant, even in lists" do
+    it "treats hidden enum values as non-existent, even in lists" do
       hidden_res_1 = auth_execute <<-GRAPHQL, context: { hide: true }
       {
         landscapeFeature(enum: TAR_PIT)
@@ -470,7 +479,7 @@ describe "GraphQL::Authorization" do
 
       assert_equal ["Argument 'enums' on Field 'landscapeFeatures' has an invalid value ([STREAM, TAR_PIT]). Expected type '[LandscapeFeature!]'."], hidden_res_2["errors"].map { |e| e["message"] }
 
-      success_res = auth_execute <<-GRAPHQL, context: { hide: false }
+      success_res = auth_execute <<-GRAPHQL, context: { hide: false, authorized: true }
       {
         landscapeFeature(enum: TAR_PIT)
         landscapeFeatures(enums: [STREAM, TAR_PIT])
@@ -500,6 +509,28 @@ describe "GraphQL::Authorization" do
       end
     end
 
+    it "rejects incoming unauthorized enum values" do
+      res = auth_execute <<-GRAPHQL, context: { }
+        {
+          landscapeFeature(enum: STREAM)
+        }
+      GRAPHQL
+
+      assert_equal ["Unauthorized LandscapeFeature: \"STREAM\""], res["errors"].map { |e| e["message"] }
+    end
+
+    it "rejects outgoing unauthorized enum values" do
+      err = assert_raises(AuthTest::LandscapeFeature::UnresolvedValueError) do
+        auth_execute <<-GRAPHQL, context: { }
+          {
+            landscapeFeature(string: "STREAM")
+          }
+        GRAPHQL
+      end
+
+      assert_equal "`Query.landscapeFeature` returned `\"STREAM\"` at `landscapeFeature`, but this value was unauthorized. Update the field or resolver to return a different value in this case (or return `nil`).", err.message
+    end
+
     it "works in introspection" do
       res = auth_execute <<-GRAPHQL, context: { hide: true, hidden_mutation: true }
         {
@@ -528,12 +559,33 @@ describe "GraphQL::Authorization" do
     end
 
     it "works when printing the SDL" do
-      full_sdl = AuthTest::Schema.to_definition
-      restricted_sdl = AuthTest::Schema.to_definition(context: { hide: true, hidden_mutation: true, hidden_relay: true })
-      assert_includes full_sdl, 'Hidden'
-      assert_includes full_sdl, 'hidden'
-      refute_includes restricted_sdl, 'Hidden'
-      refute_includes restricted_sdl, 'hidden'
+      full_sdl_lines = AuthTest::Schema.to_definition.split("\n")
+      restricted_sdl_lines = AuthTest::Schema.to_definition(context: { hide: true, hidden_mutation: true, hidden_relay: true }).split("\n")
+      expected_hidden_lines = [
+        "Autogenerated return type of DoHiddenStuff2.",
+        "type DoHiddenStuff2Payload {",
+        "Autogenerated input type of DoHiddenStuff",
+        "input DoHiddenStuffInput {",
+        "Autogenerated return type of DoHiddenStuff.",
+        "type DoHiddenStuffPayload {",
+        "interface HiddenDefaultInterface",
+        "interface HiddenInterface",
+        "type HiddenObject implements HiddenDefaultInterface & HiddenInterface {",
+        "  doHiddenStuff(",
+        "    Parameters for DoHiddenStuff",
+        "    input: DoHiddenStuffInput!",
+        "  ): DoHiddenStuffPayload",
+        "  doHiddenStuff2: DoHiddenStuff2Payload",
+        "  hidden: Int!",
+        "  hiddenConnection(",
+        "  hiddenDefaultInterface: HiddenDefaultInterface!",
+        "  hiddenEdge: RelayObjectEdge",
+        "  hiddenInterface: HiddenInterface!",
+        "  hiddenObject: HiddenObject!",
+        "  int2(hidden: Int, int: Int, unauthorized: Int): Int"
+      ]
+      assert_equal expected_hidden_lines, full_sdl_lines.select { |l| l.include?("Hidden") || l.include?("hidden") }
+      assert_equal [], restricted_sdl_lines.select { |l| l.include?("Hidden") || l.include?("hidden") }
     end
 
     it "works with directives" do
@@ -857,8 +909,11 @@ describe "GraphQL::Authorization" do
         assert_equal "Query", res["data"]["__typename"]
 
         unauth_res = auth_execute(query, context: { query_unauthorized: true })
-        assert_nil unauth_res["data"]
-        assert_equal [{"message"=>"Unauthorized Query: nil"}], unauth_res["errors"]
+
+        assert_equal({
+          "errors" => [{"message"=>"Unauthorized Query: nil"}],
+          "data" => nil,
+        }, unauth_res.to_h)
       end
 
       describe "when the object authorization raises an UnauthorizedFieldError" do
