@@ -128,8 +128,13 @@ describe GraphQL::Schema do
       it "rescues execution errors from execute_query" do
         context = {raise_execution_error: true}
         res = schema.execute(" { int(value: 2) } ", context: context)
-        assert_equal "Raised from trace execute_query", res["errors"].first["message"]
-        refute res.key?("data"), "The query doesn't run"
+
+        assert_equal({
+          "data" => nil,
+          "errors" => [
+            { "message" => "Raised from trace execute_query" },
+          ]
+        }, res.to_h)
       end
 
       it "can assign a query string there" do
@@ -166,8 +171,9 @@ describe GraphQL::Schema do
         assert multiplex_ctx[:second_instrumenter_did_begin]
         refute multiplex_ctx[:second_instrumenter_did_end]
         # No query instrumentation was run at all
-        assert_equal 0, query_1_ctx.size
-        assert_equal 0, query_2_ctx.size
+        expected_ctx_size = GraphQL::Schema.use_visibility_profile? ? 1 : 0
+        assert_equal expected_ctx_size, query_1_ctx.size
+        assert_equal expected_ctx_size, query_2_ctx.size
       end
 
       it "does full and partial query runs" do
