@@ -41,11 +41,6 @@ describe GraphQL::Execution::Interpreter do
         Box.new(value: object.sym)
       end
 
-      field :null_union_field_test, Integer, null: false
-      def null_union_field_test
-        1
-      end
-
       field :always_cached_value, Integer, null: false
       def always_cached_value
         raise "should never be called"
@@ -61,11 +56,6 @@ describe GraphQL::Execution::Interpreter do
         Query::EXPANSIONS.find { |e| e.sym == @object.expansion_sym }
       end
 
-      field :null_union_field_test, Integer
-      def null_union_field_test
-        nil
-      end
-
       field :parent_class_name, String, null: false, extras: [:parent]
 
       def parent_class_name(parent:)
@@ -143,7 +133,7 @@ describe GraphQL::Execution::Interpreter do
         base_ctx_value = context[key]
         interpreter_ctx_value = context.namespace(:interpreter)[key]
         if base_ctx_value != interpreter_ctx_value
-          raise "Context mismatch for #{key} -> #{base_ctx_value} / intepreter: #{interpreter_ctx_value}"
+          raise "Context mismatch for #{key} -> #{base_ctx_value} / interpreter: #{interpreter_ctx_value}"
         else
           base_ctx_value
         end
@@ -249,17 +239,38 @@ describe GraphQL::Execution::Interpreter do
 
     class Counter < GraphQL::Schema::Object
       field :value, Integer, null: false
+
+      def value
+        counter.value
+      end
+
       field :lazy_value, Integer, null: false
 
       def lazy_value
-        Box.new { object.value }
+        Box.new { counter.value }
       end
 
+      field :incremented_value, Integer, hash_key: :incremented_value
+
       field :increment, Counter, null: false
 
       def increment
-        object.value += 1
-        object
+        v = counter.value += 1
+        {
+          counter: counter,
+          incremented_value: v,
+        }
+      end
+
+
+      private
+
+      def counter
+        if object.is_a?(Hash) && object.key?(:counter)
+          object[:counter]
+        else
+          object
+        end
       end
     end
 
@@ -268,8 +279,11 @@ describe GraphQL::Execution::Interpreter do
 
       def increment_counter
         counter = context[:counter]
-        counter.value += 1
-        counter
+        v = counter.value += 1
+        {
+          counter: counter,
+          incremented_value: v
+        }
       end
     end
 
@@ -309,7 +323,7 @@ describe GraphQL::Execution::Interpreter do
       module EnsureThreadCleanedUp
         def execute_multiplex(multiplex:)
           res = super
-          runtime_info = Thread.current[:__graphql_runtime_info]
+          runtime_info = Fiber[:__graphql_runtime_info]
           if !runtime_info.nil? && runtime_info != {}
             if !multiplex.context[:allow_pending_thread_state]
               # `nestedQuery` can allow this
@@ -369,7 +383,7 @@ describe GraphQL::Execution::Interpreter do
       {"__typename" => "Expansion", "sym" => "RAV"},
     ]
     assert_equal expected_abstract_list, result["data"]["find"]
-    assert_nil Thread.current[:__graphql_runtime_info]
+    assert_nil Fiber[:__graphql_runtime_info]
   end
 
   it "runs a nested query and maintains proper state" do
@@ -377,18 +391,18 @@ describe GraphQL::Execution::Interpreter do
     result = InterpreterTest::Schema.execute(query_str, variables: { queryStr: "{ __typename }" })
     assert_equal '{"data":{"__typename":"Query"}}', result["data"]["nestedQuery"]["result"]
     assert_equal ["nestedQuery"], result["data"]["nestedQuery"]["currentPath"]
-    assert_nil Thread.current[:__graphql_runtime_info]
+    assert_nil Fiber[:__graphql_runtime_info]
   end
 
   it "runs mutation roots atomically and sequentially" do
     query_str = <<-GRAPHQL
     mutation {
       i1: incrementCounter { value lazyValue
-        i2: increment { value lazyValue }
-        i3: increment { value lazyValue }
+        i2: increment { value incrementedValue lazyValue }
+        i3: increment { value incrementedValue lazyValue }
       }
-      i4: incrementCounter { value lazyValue }
-      i5: incrementCounter { value lazyValue }
+      i4: incrementCounter { value incrementedValue lazyValue }
+      i5: incrementCounter { value incrementedValue lazyValue }
     }
     GRAPHQL
 
@@ -399,13 +413,13 @@ describe GraphQL::Execution::Interpreter do
         # All of these get `3` as lazy value. They're resolved together,
         # since they aren't _root_ mutation fields.
         "lazyValue" => 3,
-        "i2" => { "value" => 2, "lazyValue" => 3 },
-        "i3" => { "value" => 3, "lazyValue" => 3 },
+        "i2" => { "value" => 2, "incrementedValue" => 2, "lazyValue" => 3 },
+        "i3" => { "value" => 3, "incrementedValue" => 3, "lazyValue" => 3 },
       },
-      "i4" => { "value" => 4, "lazyValue" => 4},
-      "i5" => { "value" => 5, "lazyValue" => 5},
+      "i4" => { "value" => 4, "incrementedValue" => 4, "lazyValue" => 4},
+      "i5" => { "value" => 5, "incrementedValue" => 5, "lazyValue" => 5},
     }
-    assert_equal expected_data, result["data"]
+    assert_graphql_equal expected_data, result["data"]
   end
 
   it "runs skip and include" do
@@ -427,16 +441,8 @@ describe GraphQL::Execution::Interpreter do
       "exp3" => {"name" => "Ravnica, City of Guilds"},
       "exp5" => {"name" => "Ravnica, City of Guilds"},
     }
-    assert_equal expected_data, result["data"]
-    assert_nil Thread.current[:__graphql_runtime_info]
-  end
-
-  describe "temporary interpreter flag" do
-    it "is set" do
-      # This can be removed later, just a sanity check during migration
-      res = InterpreterTest::Schema.execute("{ __typename }")
-      assert_equal true, res.context.interpreter?
-    end
+    assert_graphql_equal expected_data, result["data"]
+    assert_nil Fiber[:__graphql_runtime_info]
   end
 
   describe "runtime info in context" do
@@ -477,7 +483,7 @@ describe GraphQL::Execution::Interpreter do
       # propagated to here
       assert_nil res["data"].fetch("expansion")
       assert_equal ["Cannot return null for non-nullable field Expansion.name"], res["errors"].map { |e| e["message"] }
-      assert_nil Thread.current[:__graphql_runtime_info]
+      assert_nil Fiber[:__graphql_runtime_info]
     end
 
     it "places errors ahead of data in the response" do
@@ -519,7 +525,7 @@ describe GraphQL::Execution::Interpreter do
         }
       }
       GRAPHQL
-      assert_equal ["Cannot return null for non-nullable field Query.find"], res["errors"].map { |e| e["message"] }
+      assert_equal ["Cannot return null for non-nullable element of type 'Entity!' for Query.find"], res["errors"].map { |e| e["message"] }
     end
 
     it "works with lists of unions" do
@@ -535,30 +541,13 @@ describe GraphQL::Execution::Interpreter do
 
       assert_equal 3, res["data"]["findMany"].size
       assert_equal "RAV", res["data"]["findMany"][0]["sym"]
-      assert_equal nil, res["data"]["findMany"][1]
-      assert_equal nil, res["data"]["findMany"][2]
+      assert_nil res["data"]["findMany"][1]
+      assert_nil res["data"]["findMany"][2]
       assert_equal false, res.key?("errors")
 
       assert_equal Hash, res["data"].class
       assert_equal Array, res["data"]["findMany"].class
     end
-
-    it "works with union lists that have members of different kinds, with different nullabilities" do
-      res = InterpreterTest::Schema.execute <<-GRAPHQL
-      {
-        findMany(ids: ["RAV", "Dark Confidant"]) {
-          ... on Expansion {
-            nullUnionFieldTest
-          }
-          ... on Card {
-            nullUnionFieldTest
-          }
-        }
-      }
-      GRAPHQL
-
-      assert_equal [1, nil], res["data"]["findMany"].map { |f| f["nullUnionFieldTest"] }
-    end
   end
 
   describe "duplicated fields" do
@@ -869,9 +858,6 @@ describe GraphQL::Execution::Interpreter do
 
       result = RaisedErrorSchema.execute(querystring)
       expected_result = {
-        "data" => {
-          "iface" => { "txn" => nil, "msg" => "THIS SHOULD SHOW UP" },
-        },
         "errors" => [
           {
             "message"=>"boom",
@@ -879,8 +865,11 @@ describe GraphQL::Execution::Interpreter do
             "path"=>["iface", "txn", "fails"]
           },
         ],
+        "data" => {
+          "iface" => { "txn" => nil, "msg" => "THIS SHOULD SHOW UP" },
+        },
       }
-      assert_equal expected_result, result.to_h
+      assert_graphql_equal expected_result, result.to_h
     end
   end
 
@@ -957,4 +946,25 @@ describe GraphQL::Execution::Interpreter do
       assert_equal(expected_result, res["data"])
     end
   end
+
+  describe "multiplex queries" do
+    it "runs multiplex queries" do
+      result = InterpreterTest::Schema.multiplex([
+        {
+          query: "query Card($name: String!) { card(name: $name) { colors } }",
+          variables: { name: "Dark Confidant" },
+          operation_name: "Card"
+        },
+        {
+          query: "query Expansion($expansion: String!) { expansion(sym: $expansion) { cards { name } } }",
+          variables: { expansion: "RAV" },
+          operation_name: "Expansion"
+        }
+      ])
+
+      assert_equal ["BLACK"], result[0]["data"]["card"]["colors"]
+      assert_equal [{"name" => "Dark Confidant"}], result[1]["data"]["expansion"]["cards"]
+      assert_nil Fiber[:__graphql_runtime_info]
+    end
+  end
 end
