@@ -74,7 +74,7 @@ describe GraphQL::Execution::Lazy do
         ],
       }
 
-      assert_equal expected_data, res["data"]
+      assert_graphql_equal expected_data, res["data"]
     end
 
     [
@@ -101,7 +101,7 @@ describe GraphQL::Execution::Lazy do
       end
     end
 
-    it "Handles fields that return nil" do
+    it "Handles fields that return nil and batches lazy resultion across depths when possible" do
       values = [
         LazyHelpers::MAGIC_NUMBER_THAT_RETURNS_NIL,
         LazyHelpers::MAGIC_NUMBER_WITH_LAZY_AUTHORIZED_HOOK,
@@ -176,7 +176,7 @@ describe GraphQL::Execution::Lazy do
         "b" => nil,
         "c" => { "value" => 3 },
       }
-      assert_equal expected_data, res["data"]
+      assert_graphql_equal expected_data, res["data"]
 
       expected_errors = [{
         "message"=>"13 is unlucky",
