@@ -935,7 +935,7 @@ describe GraphQL::Query do
       end
     end
 
-    warden = GraphQL::Schema::Warden.new(schema: schema_class, context: nil)
+    warden = GraphQL::Schema::Warden.new(schema: schema_class, context: {})
     res = Jazz::Schema.execute("{ __typename } ", warden: warden)
     assert_equal ["Schema is not configured for queries"], res["errors"].map { |e| e["message"] }
   end
@@ -1091,7 +1091,7 @@ describe GraphQL::Query do
   describe "using GraphQL.default_parser" do
     module DummyParser
       DOC = GraphQL::Language::Parser.parse("{ __typename }")
-      def self.parse(query_str, trace: nil, filename: nil)
+      def self.parse(query_str, trace: nil, filename: nil, max_tokens: nil)
         DOC
       end
     end
