@@ -18,15 +18,12 @@ schema {
   query: HelloScalars
 }
 
-type EmptyType
-
 type HelloScalars {
   bool: Boolean
   float: Float
   id: ID
   int: Int
   str: String!
-  t: EmptyType
 }
       SCHEMA
 
@@ -904,8 +901,8 @@ enum Enum {
 }
 
 type Query {
-  field(argument: String): String
-  deprecatedField(argument: String): String @deprecated(reason: "Test")
+  field(argument: Enum): Interface
+  deprecatedField(argument: Input): Union @deprecated(reason: "Test")
 }
 
 interface Interface {
@@ -928,7 +925,7 @@ directive @Directive (
 directive @custom(thing: Boolean) on SCHEMA
 
 type Type implements Interface {
-  field(argument: String): String
+  field(argument: Scalar): Type
 }
       GRAPHQL
 
@@ -941,8 +938,8 @@ type Type implements Interface {
       assert_equal [9, 1], schema.types["Query"].ast_node.position
       assert_equal [10, 3], schema.types["Query"].fields["field"].ast_node.position
       assert_equal [10, 9], schema.types["Query"].fields["field"].arguments["argument"].ast_node.position
-      assert_equal [11, 45], schema.types["Query"].fields["deprecatedField"].ast_node.directives[0].position
-      assert_equal [11, 57], schema.types["Query"].fields["deprecatedField"].ast_node.directives[0].arguments[0].position
+      assert_equal [11, 43], schema.types["Query"].fields["deprecatedField"].ast_node.directives[0].position
+      assert_equal [11, 55], schema.types["Query"].fields["deprecatedField"].ast_node.directives[0].arguments[0].position
       assert_equal [14, 1], schema.types["Interface"].ast_node.position
       assert_equal [15, 3], schema.types["Interface"].fields["field"].ast_node.position
       assert_equal [15, 9], schema.types["Interface"].fields["field"].arguments["argument"].ast_node.position
@@ -1461,7 +1458,7 @@ type ThingEdge {
             }
           }
         }
-        assert_equal expected_data, result["data"]
+        assert_graphql_equal expected_data, result["data"]
       end
 
       it "doesn't add arguments that aren't in the IDL" do
@@ -1485,7 +1482,7 @@ type ThingEdge {
     GRAPHQL
 
     schema = GraphQL::Schema.from_definition(schema_str)
-    assert_equal schema.directives["myDirective"].get_argument("id").type, schema.find("ID")
+    assert_equal schema.directives["myDirective"].get_argument("id").type, schema.get_type("ID")
   end
 
   describe "orphan types" do
@@ -1752,4 +1749,192 @@ type ReachableType implements Node {
     schema = GraphQL::Schema.from_definition(schema_str)
     assert_equal schema_str, schema.to_definition
   end
+
+  describe "Custom base types" do
+    class MyObject < GraphQL::Schema::Object; end
+    class MyUnion < GraphQL::Schema::Union; end
+    class MyScalar < GraphQL::Schema::Scalar; end
+    class MyEnum < GraphQL::Schema::Enum; end
+    class MyInputObject < GraphQL::Schema::InputObject; end
+    module MyInterface
+      include GraphQL::Schema::Interface
+    end
+
+    it "Builds a schema using custom base types" do
+      schema = GraphQL::Schema.from_definition(%|
+        interface A { a:Int }
+        type B implements A { a:Int }
+        union C = B
+        scalar D
+        enum E { YES }
+        input F { f:Int }
+        type Query {
+          a(f:F): A
+          c: C
+          d: D
+          e: E
+        }
+      |, base_types: {
+        object: MyObject,
+        interface: MyInterface,
+        union: MyUnion,
+        scalar: MyScalar,
+        enum: MyEnum,
+        input_object: MyInputObject,
+      })
+
+      assert schema.get_type("A") <= MyInterface
+      assert schema.get_type("B") <= MyObject
+      assert schema.get_type("C") <= MyUnion
+      assert schema.get_type("D") <= MyScalar
+      assert schema.get_type("E") <= MyEnum
+      assert schema.get_type("F") <= MyInputObject
+    end
+  end
+
+  describe "Type extensions" do
+    it "Extends object types with fields and directives" do
+      sdl = %|
+        directive @a on OBJECT
+        directive @b on OBJECT
+        type T @a { a:Int }
+        extend type T @b { b:Int }
+        type Query { t:T }
+      |
+      schema = GraphQL::Schema.from_definition(sdl)
+      assert_equal ["a", "b"], schema.get_type("T").fields.keys.sort
+      assert_equal ["a", "b"], schema.get_type("T").directives.map(&:graphql_name).sort
+    end
+
+    it "Extends object types with interfaces" do
+      sdl = %|
+        interface A { a:Int }
+        interface B { b:Int }
+        type T implements A { a:Int }
+        extend type T implements B { b:Int }
+        type Query { t:T }
+      |
+      schema = GraphQL::Schema.from_definition(sdl)
+      assert_equal ["A", "B"], schema.get_type("T").interfaces.map(&:graphql_name).sort
+    end
+
+    it "Extends interface types with fields and directives" do
+      sdl = %|
+        directive @a on INTERFACE
+        directive @b on INTERFACE
+        interface I @a { a:Int }
+        extend interface I @b { b:Int }
+        type T implements I { a:Int b:Int }
+        type Query { t:T }
+      |
+      schema = GraphQL::Schema.from_definition(sdl)
+      assert_equal ["a", "b"], schema.get_type("I").fields.keys.sort
+      assert_equal ["a", "b"], schema.get_type("I").directives.map(&:graphql_name).sort
+    end
+
+    it "Extends interface types with interfaces" do
+      sdl = %|
+        interface A { a:Int }
+        interface B { b:Int }
+        extend interface A implements B { b:Int }
+        type T implements A { a:Int b:Int }
+        type Query { t:T }
+      |
+      schema = GraphQL::Schema.from_definition(sdl)
+      assert_equal ["B"], schema.get_type("A").interfaces.map(&:graphql_name).sort
+    end
+
+    it "Extends scalar types with directives" do
+      sdl = %|
+        directive @a on SCALAR
+        directive @b on SCALAR
+        scalar T @a
+        extend scalar T @b
+        type Query { t:T }
+      |
+      schema = GraphQL::Schema.from_definition(sdl)
+      assert_equal ["a", "b"], schema.get_type("T").directives.map(&:graphql_name).sort
+    end
+
+    it "Extends enum types with values and directives" do
+      sdl = %|
+        directive @a on ENUM
+        directive @b on ENUM
+        enum T @a { YES }
+        extend enum T @b { NO }
+        type Query { t:T }
+      |
+      schema = GraphQL::Schema.from_definition(sdl)
+      assert_equal ["NO", "YES"], schema.get_type("T").values.keys.sort
+      assert_equal ["a", "b"], schema.get_type("T").directives.map(&:graphql_name).sort
+    end
+
+    it "Extends input object types with arguments and directives" do
+      sdl = %|
+        directive @a on INPUT_OBJECT
+        directive @b on INPUT_OBJECT
+        input T @a { a:Int }
+        extend input T @b { b:Int }
+        type Query { t:T }
+      |
+      schema = GraphQL::Schema.from_definition(sdl)
+      assert_equal ["a", "b"], schema.get_type("T").arguments.keys.sort
+      assert_equal ["a", "b"], schema.get_type("T").directives.map(&:graphql_name).sort
+    end
+
+    it "Extends union types with types and directives" do
+      sdl = %|
+        directive @a on UNION
+        directive @b on UNION
+        type A { a:Int }
+        type B { b:Int }
+        union T @a = A
+        extend union T @b = B
+        type Query { t:T }
+      |
+      schema = GraphQL::Schema.from_definition(sdl)
+      assert_equal ["A", "B"], schema.possible_types(schema.get_type("T")).map(&:graphql_name).sort
+      assert_equal ["a", "b"], schema.get_type("T").directives.map(&:graphql_name).sort
+    end
+  end
+
+  if USING_C_PARSER
+    it "makes frozen identifiers with CParser" do
+      schema_class = GraphQL::Schema.from_definition("type Query { f: Boolean }")
+      assert_equal "Query", schema_class.query.ast_node.name
+      assert schema_class.query.ast_node.name.frozen?
+    end
+  end
+
+  it "works when directives use argument types defined after them" do
+    schema_sdl = <<~GRAPHQL
+      directive @foo(value: SomeEnum!) on OBJECT
+      directive @bar on ENUM_VALUE
+
+      enum SomeEnum {
+        BAZ @bar
+      }
+
+      type Query {
+        someEnum: SomeEnum!
+      }
+    GRAPHQL
+    GraphQL::Schema.from_definition(schema_sdl)
+  end
+
+  it "works with invalid directive argument value" do
+    sdl = <<~EOS
+      directive @requiresScopes(scopes: [[String]]) on FIELD_DEFINITION
+
+      type Query {
+        product: Product
+      }
+
+      type Product {
+        shippingEstimate: String @requiresScopes(scopes: "shipping")
+      }
+    EOS
+
+    assert GraphQL::Schema.from_definition(sdl).to_definition
+  end
 end
