@@ -1,5 +1,6 @@
 # frozen_string_literal: true
 require "spec_helper"
+
 describe GraphQL::Schema::Field do
   describe "graphql definition" do
     let(:object_class) { Jazz::Query }
@@ -30,9 +31,10 @@ describe GraphQL::Schema::Field do
     it "camelizes the field name, unless camelize: false" do
       assert_equal 'inspectInput', field.name
 
-      underscored_field = GraphQL::Schema::Field.from_options(:underscored_field, String, null: false, camelize: false, owner: nil) do
+      example_obj = Class.new(GraphQL::Schema::Object) { graphql_name("Example") }
+      underscored_field = example_obj.field(:underscored_field, String, null: false, camelize: false, owner: nil) do
         argument :underscored_arg, String, camelize: false
-      end
+      end.ensure_loaded
 
       arg_name, arg_defn = underscored_field.arguments.first
       assert_equal 'underscored_arg', arg_name
@@ -54,17 +56,71 @@ describe GraphQL::Schema::Field do
     end
 
     it "accepts a block for definition" do
+      field_defn = nil
       object = Class.new(Jazz::BaseObject) do
         graphql_name "JustAName"
 
-        field :test, String do
+        field_defn = field :test do
           argument :test, String
           description "A Description."
+          comment "A Comment."
+          type String
         end
       end
 
+      assert_nil field_defn.description, "The block isn't called right away"
+      assert_nil field_defn.type, "The block isn't called right away"
+      field_defn.ensure_loaded
+      assert_equal "String", field_defn.type.graphql_name
+
       assert_equal "test", object.fields["test"].arguments["test"].name
       assert_equal "A Description.", object.fields["test"].description
+      assert_equal "A Comment.", object.fields["test"].comment
+    end
+
+    it "sets connection? when type is given in a block" do
+      field_defn = nil
+      Class.new(Jazz::BaseObject) do
+        graphql_name "JustAName"
+
+        field_defn = field :instruments do
+          type Jazz::InstrumentType.connection_type
+        end
+      end
+
+      assert_equal false, field_defn.connection?
+      assert_equal false, field_defn.scoped?
+      assert_equal [], field_defn.extensions
+      field_defn.ensure_loaded
+      assert_equal true, field_defn.scoped?
+      assert_equal true, field_defn.connection?
+      assert_equal [GraphQL::Schema::Field::ScopeExtension, GraphQL::Schema::Field::ConnectionExtension], field_defn.extensions.map(&:class)
+
+      class LazyConnectionSchema < GraphQL::Schema
+        module Entity
+          include GraphQL::Schema::Interface
+          field :things, description: "THINGS-DESCRIPTION" do
+            type Thing.connection_type
+          end
+        end
+
+        class Thing < GraphQL::Schema::Object
+          field :name, String
+          implements Entity
+        end
+
+        class Query < GraphQL::Schema::Object
+          field :entity, Entity
+        end
+        query(Query)
+        orphan_types Thing
+      end
+
+      defn_str = LazyConnectionSchema.to_definition
+      assert_includes defn_str, "ThingConnection"
+      assert_includes defn_str, "first: Int"
+      assert_includes defn_str, "THINGS-DESCRIPTION"
+      assert_equal "ThingConnection", LazyConnectionSchema.get_field("Entity", "things").type.graphql_name
     end
 
     it "accepts a block for definition and yields the field if the block has an arity of one" do
@@ -74,18 +130,21 @@ describe GraphQL::Schema::Field do
         field :test, String do |field|
           field.argument :test, String
           field.description "A Description."
+          field.comment "A Comment."
         end
       end
 
       assert_equal "test", object.fields["test"].arguments["test"].name
       assert_equal "A Description.", object.fields["test"].description
+      assert_equal "A Comment.", object.fields["test"].comment
     end
 
     it "accepts anonymous classes as type" do
       type = Class.new(GraphQL::Schema::Object) do
         graphql_name 'MyType'
       end
-      field = GraphQL::Schema::Field.from_options(:my_field, type, owner: nil, null: true)
+      example_obj = Class.new(GraphQL::Schema::Object) { graphql_name("Example") }
+      field = example_obj.field(:my_field, type, owner: nil, null: true)
       assert_equal type, field.type
     end
 
@@ -345,7 +404,7 @@ describe GraphQL::Schema::Field do
 
             field :complexityTest, String do
               complexity 'One hundred and eighty'
-            end
+            end.ensure_loaded
           end
         end
 
@@ -359,7 +418,7 @@ describe GraphQL::Schema::Field do
 
             field :complexityTest, String do
               complexity ->(one, two) { 52 }
-            end
+            end.ensure_loaded
           end
         end
 
@@ -444,7 +503,8 @@ describe GraphQL::Schema::Field do
 
   describe "#original_name" do
     it "is exactly the same as the passed in name" do
-      field = GraphQL::Schema::Field.from_options(
+      example_obj = Class.new(GraphQL::Schema::Object) { graphql_name("Example") }
+      field = example_obj.field(
         :my_field,
         String,
         null: false,
@@ -584,7 +644,7 @@ describe GraphQL::Schema::Field do
           { "name" => "John Cleese" }
         ]
       }
-      assert_equal expected_result, result
+      assert_graphql_equal expected_result, result
     end
   end
 
@@ -648,18 +708,18 @@ describe GraphQL::Schema::Field do
 
       search_results = res["data"]["searchResults"]
       expected_result = {
+        "method" => "hash-key-works-when-underlying-object-responds-to-field-name",
         "lowercase" => "lowercase-works",
         "Capital" => "capital-camelize-false-works",
         "Other" => "capital-camelize-true-works",
         "OtherCapital" => "explicit-hash-key-works",
-        "method" => "hash-key-works-when-underlying-object-responds-to-field-name",
         "stringifiedHashKey" => "hash-key-is-tried-as-string",
         "booleanTrueWithHashKey" => true,
         "booleanFalseWithHashKey" => false,
         "booleanFalseWithSymbolizedHashKey" => false
 
       }
-      assert_equal expected_result, search_results
+      assert_graphql_equal expected_result, search_results
     end
 
     it "works with non-hash instances" do
@@ -681,17 +741,17 @@ describe GraphQL::Schema::Field do
 
       search_results = res["data"]["ostructResults"]
       expected_result = {
+        "method" => "hash-key-works-when-underlying-object-responds-to-field-name",
         "lowercase" => "lowercase-works",
         "Capital" => "capital-camelize-false-works",
         "Other" => "capital-camelize-true-works",
         "OtherCapital" => "explicit-hash-key-works",
-        "method" => "hash-key-works-when-underlying-object-responds-to-field-name",
         "stringifiedHashKey" => "hash-key-is-tried-as-string",
         "booleanTrueWithHashKey" => true,
         "booleanFalseWithHashKey" => false,
         "booleanFalseWithSymbolizedHashKey" => false
       }
-      assert_equal expected_result, search_results
+      assert_graphql_equal expected_result, search_results
     end
 
     it "populates `method_str`" do
@@ -718,6 +778,7 @@ This is probably a bug in GraphQL-Ruby, please report this error on GitHub: http
   it "Delegates many properties to its @resolver_class" do
     resolver = Class.new(GraphQL::Schema::Resolver) do
       description "description 1"
+      comment "comment 1"
       type [GraphQL::Types::Float], null: true
 
       argument :b, GraphQL::Types::Float
@@ -726,8 +787,10 @@ This is probably a bug in GraphQL-Ruby, please report this error on GitHub: http
     field = GraphQL::Schema::Field.new(name: "blah", owner: nil, resolver_class: resolver, extras: [:blah]) do
       argument :a, GraphQL::Types::Int
     end
+    field.ensure_loaded
 
     assert_equal "description 1", field.description
+    assert_equal "comment 1", field.comment
     assert_equal "[Float!]", field.type.to_type_signature
     assert_equal 1, field.complexity
     assert_equal :resolve_with_support, field.resolver_method
@@ -739,6 +802,7 @@ This is probably a bug in GraphQL-Ruby, please report this error on GitHub: http
     assert_equal true, field.scoped?
 
     resolver.description("description 2")
+    resolver.comment("comment 2")
     resolver.type(GraphQL::Types::String, null: false)
     resolver.complexity(5)
     resolver.resolver_method(:blah)
@@ -748,6 +812,7 @@ This is probably a bug in GraphQL-Ruby, please report this error on GitHub: http
     resolver.argument(:c, GraphQL::Types::Boolean)
 
     assert_equal "description 2", field.description
+    assert_equal "comment 2", field.comment
     assert_equal "String!", field.type.to_type_signature
     assert_equal 5, field.complexity
     assert_equal :blah, field.resolver_method
@@ -799,7 +864,10 @@ This is probably a bug in GraphQL-Ruby, please report this error on GitHub: http
     shapes = Set.new
 
     # This is custom state added by some test schemas:
-    custom_ivars = [:@upcase, :@future_schema, :@visible, :@allow_for, :@metadata]
+    custom_ivars = [:@upcase, :@future_schema, :@visible, :@allow_for, :@metadata, :@admin_only]
+
+    # Remove any invalid (non-retained) field instances from the heap
+    GC.start
 
     ObjectSpace.each_object(GraphQL::Schema::Field) do |field_obj|
       field_ivars = field_obj.instance_variables
@@ -858,4 +926,35 @@ This is probably a bug in GraphQL-Ruby, please report this error on GitHub: http
     field = GraphQL::Schema::Field.new(name: "blah", owner: nil, type: FieldConnectionTest::Connection, connection: true)
     assert field.connection?
   end
+
+  describe "argument documentation" do
+    it "HasFields::field documents each argument" do
+      has_fields_field_comment = File.read("./lib/graphql/schema/member/has_fields.rb")[/(\s+#[^\n]*\n)+\s+def field\(/m]
+      has_field_field_doc_param_names = has_fields_field_comment.split("\n").map do |line|
+        line[/@param (\S+)/] || line[/@option kwargs \[.*\] :(\S+)/]
+        $1
+      end.compact
+
+      field_initialize_argument_names = GraphQL::Schema::Field.instance_method(:initialize).parameters.map { |param| param[1].to_s }
+
+      expected_differences = [
+        "name_positional",
+        "type_positional",
+        "desc_positional",
+        "mutation",
+        "resolver",
+        "subscription",
+        "kwargs",
+      ]
+      assert_equal expected_differences, has_field_field_doc_param_names - field_initialize_argument_names
+      assert_equal ["owner", "resolver_class"], field_initialize_argument_names - has_field_field_doc_param_names
+    end
+
+    it "Field::initialize documents each argument" do
+      field_initialize_comment = File.read("./lib/graphql/schema/field.rb")[/(\s+#[^\n]*\n)+ {6}def initialize\(/m]
+      field_initialize_doc_param_names = field_initialize_comment.split("\n").map { |line| line[/@param (\S+)/]; $1 }.compact
+      field_initialize_argument_names = GraphQL::Schema::Field.instance_method(:initialize).parameters.map { |param| param[1].to_s }
+      assert_equal field_initialize_doc_param_names.sort, field_initialize_argument_names.sort
+    end
+  end
 end
