@@ -158,6 +158,25 @@ describe GraphQL::Schema::Interface do
     end
   end
 
+  describe "comments" do
+    class SchemaWithInterface < GraphQL::Schema
+      module InterfaceWithComment
+        include GraphQL::Schema::Interface
+        comment "Interface comment"
+      end
+
+      class Query < GraphQL::Schema::Object
+        implements InterfaceWithComment
+      end
+
+      query(Query)
+    end
+
+    it "assigns comment to the interface" do
+      assert_equal("Interface comment", SchemaWithInterface::Query.interfaces[0].comment)
+    end
+  end
+
   describe "can implement other interfaces" do
     class InterfaceImplementsSchema < GraphQL::Schema
       module InterfaceA
@@ -226,7 +245,7 @@ type Query implements InterfaceA & InterfaceB {
     end
   end
 
-  describe "transitive implemention of same interface twice" do
+  describe "transitive implementation of same interface twice" do
     class TransitiveInterfaceSchema < GraphQL::Schema
       module Node
         include GraphQL::Schema::Interface
@@ -340,6 +359,11 @@ interface Timestamped implements Node {
 
       assert_equal ["Named", "Node", "Timestamped"], interfaces_names
     end
+
+    it "doesn't return interfaces as possible types" do
+      pts = TransitiveInterfaceSchema.possible_types(TransitiveInterfaceSchema::Node)
+      assert_equal ["Thing"], pts.map(&:graphql_name)
+    end
   end
 
   describe "supplying a fallback_value to a field" do
@@ -482,8 +506,8 @@ interface Timestamped implements Node {
     let(:interface) { Dummy::Edible }
 
     it "has possible types" do
-      expected_defns = [Dummy::Cheese, Dummy::Milk, Dummy::Honey, Dummy::Aspartame]
-      assert_equal(expected_defns, Dummy::Schema.possible_types(interface))
+      expected_defns = [Dummy::Aspartame, Dummy::Cheese, Dummy::Honey, Dummy::Milk]
+      assert_equal(expected_defns, Dummy::Schema.possible_types(interface).sort_by(&:graphql_name))
     end
 
     describe "query evaluation" do
@@ -499,7 +523,7 @@ interface Timestamped implements Node {
       end
     end
 
-    describe "mergable query evaluation" do
+    describe "mergeable query evaluation" do
       let(:result) { Dummy::Schema.execute(query_string, variables: {"cheeseId" => 2})}
       let(:query_string) {%|
         query fav {
@@ -557,7 +581,7 @@ interface Timestamped implements Node {
             {"__typename"=>"Milk", "origin"=>"Antiquity"},
           ]
 
-          assert_equal expected_data, result["data"]["allEdible"]
+          assert_graphql_equal expected_data, result["data"]["allEdible"]
         end
       end
     end
@@ -603,7 +627,7 @@ interface Timestamped implements Node {
             {"__typename"=>"Milk", "fatContent"=>0.04}
           ]
         }
-        assert_equal expected_result, result["data"]
+        assert_graphql_equal expected_result, result["data"]
       end
 
       describe "in definition_methods when implementing another interface" do
@@ -612,7 +636,7 @@ interface Timestamped implements Node {
             include GraphQL::Schema::Interface
             definition_methods do
               def resolve_type(obj, ctx)
-                raise "This should never be called -- it's overriden"
+                raise "This should never be called -- it's overridden"
               end
             end
           end
@@ -662,4 +686,90 @@ interface Timestamped implements Node {
       end
     end
   end
+
+  describe ".comment" do
+    it "isn't inherited" do
+      int1 = Module.new do
+        include GraphQL::Schema::Interface
+        graphql_name "Int1"
+        comment "TODO: fix this"
+      end
+
+      int2 = Module.new do
+        include int1
+        graphql_name "Int2"
+      end
+
+      assert_equal "TODO: fix this", int1.comment
+      assert_nil int2.comment
+    end
+  end
+
+  describe "when used as loads" do
+    class InterfaceLoadsSchema < GraphQL::Schema
+      module Sharpenable
+        include GraphQL::Schema::Interface
+        field :is_sharp, Boolean
+      end
+
+      class Pencil < GraphQL::Schema::Object
+        implements Sharpenable
+        field :color, String
+      end
+
+      class Chisel < GraphQL::Schema::Object
+        implements Sharpenable
+        field :width, Float
+      end
+
+      class Mallet < GraphQL::Schema::Object
+        field :weight, Float
+      end
+
+      class Tool < GraphQL::Schema::Union
+        possible_types(Mallet, Chisel, Pencil)
+      end
+
+      class Query < GraphQL::Schema::Object
+        field :tool, Tool do
+          argument :id, ID, loads: Sharpenable, as: :sharpenable_object
+        end
+
+        def tool(sharpenable_object:)
+          sharpenable_object
+        end
+      end
+
+      query(Query)
+      if ADD_WARDEN
+        use GraphQL::Schema::Warden
+      else
+        use GraphQL::Schema::Visibility
+      end
+
+      def self.resolve_type(abs_type, obj, ctx)
+        obj[:object_type]
+      end
+
+      def self.object_from_id(id, ctx)
+        case id
+        when "chisel"
+          { width: 5, is_sharp: true, object_type: Chisel }
+        when "pencil"
+          { color: "gray", is_sharp: false, object_type: Pencil }
+        when "mallet"
+          { weight: 16, object_type: Mallet }
+        else
+          nil
+        end
+      end
+    end
+
+    it "typechecks the loaded object" do
+      assert_equal({ "tool" => { "__typename" => "Chisel" } }, InterfaceLoadsSchema.execute("{ tool(id: \"chisel\") { __typename } }")["data"])
+      assert_equal({ "tool" => { "__typename" => "Pencil" } }, InterfaceLoadsSchema.execute("{ tool(id: \"pencil\") { __typename } }")["data"])
+
+      assert_equal(["No object found for `id: \"mallet\"`"], InterfaceLoadsSchema.execute("{ tool(id: \"mallet\") { __typename } }")["errors"].map {|e| e["message"]})
+    end
+  end
 end
