@@ -44,10 +44,11 @@ describe GraphQL::Schema do
         cursor_encoder Object.new
         context_class Class.new
         directives [DummyFeature1]
-        tracer GraphQL::Tracing::DataDogTracing
         extra_types ExtraType
         query_analyzer Object.new
         multiplex_analyzer Object.new
+        validate_timeout 100
+        max_query_string_tokens 500
         rescue_from(StandardError) { }
         use GraphQL::Backtrace
         use GraphQL::Subscriptions::ActionCableSubscriptions, action_cable: nil, action_cable_coder: JSON
@@ -69,11 +70,16 @@ describe GraphQL::Schema do
       assert_equal base_schema.orphan_types, schema.orphan_types
       assert_equal base_schema.context_class, schema.context_class
       assert_equal base_schema.directives, schema.directives
-      assert_equal base_schema.tracers, schema.tracers
+      assert_equal base_schema.max_query_string_tokens, schema.max_query_string_tokens
       assert_equal base_schema.query_analyzers, schema.query_analyzers
       assert_equal base_schema.multiplex_analyzers, schema.multiplex_analyzers
       assert_equal base_schema.disable_introspection_entry_points?, schema.disable_introspection_entry_points?
-      assert_equal [GraphQL::Backtrace, GraphQL::Subscriptions::ActionCableSubscriptions], schema.plugins.map(&:first)
+      expected_plugins = [
+        (GraphQL::Schema.use_visibility_profile? ? GraphQL::Schema::Visibility : nil),
+        GraphQL::Backtrace,
+        GraphQL::Subscriptions::ActionCableSubscriptions
+      ].compact
+      assert_equal expected_plugins, schema.plugins.map(&:first)
       assert_equal [ExtraType], base_schema.extra_types
       assert_equal [ExtraType], schema.extra_types
       assert_instance_of GraphQL::Subscriptions::ActionCableSubscriptions, schema.subscriptions
@@ -87,6 +93,7 @@ describe GraphQL::Schema do
         use CustomSubscriptions, action_cable: nil, action_cable_coder: JSON
         query_class(custom_query_class)
         extra_types [extra_type_2]
+        max_query_string_tokens nil
       end
 
       query = Class.new(GraphQL::Schema::Object) do
@@ -123,13 +130,13 @@ describe GraphQL::Schema do
       multiplex_analyzer = Object.new
       schema.multiplex_analyzer(multiplex_analyzer)
       schema.rescue_from(GraphQL::ExecutionError)
-      schema.tracer(GraphQL::Tracing::NewRelicTracing)
 
       assert_equal query, schema.query
       assert_equal mutation, schema.mutation
       assert_equal subscription, schema.subscription
       assert_equal introspection, schema.introspection
       assert_equal cursor_encoder, schema.cursor_encoder
+      assert_nil schema.max_query_string_tokens
 
       assert_equal context_class, schema.context_class
       assert_equal 10, schema.validate_timeout
@@ -141,17 +148,48 @@ describe GraphQL::Schema do
       assert_equal schema.directives, GraphQL::Schema.default_directives.merge(DummyFeature1.graphql_name => DummyFeature1, DummyFeature2.graphql_name => DummyFeature2)
       assert_equal base_schema.query_analyzers + [query_analyzer], schema.query_analyzers
       assert_equal base_schema.multiplex_analyzers + [multiplex_analyzer], schema.multiplex_analyzers
-      assert_equal [GraphQL::Backtrace, GraphQL::Subscriptions::ActionCableSubscriptions, CustomSubscriptions], schema.plugins.map(&:first)
-      assert_equal [GraphQL::Tracing::DataDogTracing], base_schema.tracers
-      assert_includes base_schema.new_trace.class.ancestors, GraphQL::Tracing::CallLegacyTracers
-      assert_equal [GraphQL::Tracing::DataDogTracing, GraphQL::Tracing::NewRelicTracing], schema.tracers
-      assert_includes schema.new_trace.class.ancestors, GraphQL::Tracing::CallLegacyTracers
+      expected_plugins = [GraphQL::Backtrace, GraphQL::Subscriptions::ActionCableSubscriptions, CustomSubscriptions]
+      if GraphQL::Schema.use_visibility_profile?
+        expected_plugins.unshift(GraphQL::Schema::Visibility)
+      end
+      assert_equal expected_plugins, schema.plugins.map(&:first)
       assert_equal custom_query_class, schema.query_class
       assert_equal [ExtraType, extra_type_2], schema.extra_types
       assert_instance_of CustomSubscriptions, schema.subscriptions
     end
   end
 
+  class ExampleOptionEnum < GraphQL::Schema::Enum
+  end
+  it "rejects non-object types to orphan_types" do
+    object_type = Class.new(GraphQL::Schema::Object)
+    err = assert_raises ArgumentError do
+      Class.new(GraphQL::Schema) do
+        orphan_types(ExampleOptionEnum, object_type)
+      end
+    end
+
+    expected_msg = "Only object type classes should be added as `orphan_types(...)`.
+
+- Remove these no-op types from `orphan_types`: ExampleOptionEnum (ENUM)
+- See https://graphql-ruby.org/type_definitions/interfaces.html#orphan-types
+
+To add other types to your schema, you might want `extra_types`: https://graphql-ruby.org/schema/definition.html#extra-types
+"
+    assert_equal expected_msg, err.message
+  end
+
+  describe ".references_to" do
+    it "doesn't include any duplicates" do
+      [Dummy::Schema, Jazz::Schema].each do |schema_class|
+        schema_class.references_to.each do |referent, references|
+          ref_paths = references.map { |r| "#{r.class}/#{r.path}"}.sort
+          assert_equal ref_paths.uniq, ref_paths, "#{schema_class}.references_to has unique entries for `#{referent}`"
+        end
+      end
+    end
+  end
+
   describe "merged, inherited caches" do
     METHODS_TO_CACHE = {
       types: 1,
@@ -167,7 +205,7 @@ describe GraphQL::Schema do
         end
 
         METHODS_TO_CACHE.each do |method_name, allowed_calls|
-          define_singleton_method(method_name) do |*args, &block|
+          define_singleton_method(method_name) do |*args, **kwargs, &block|
             if @calls
               call_count = @calls[method_name] += 1
               @callers[method_name] << caller
@@ -177,7 +215,7 @@ describe GraphQL::Schema do
             if call_count > allowed_calls
               raise "Called #{method_name} more than #{allowed_calls} times, previous caller: \n#{@callers[method_name].first.join("\n")}"
             end
-            super(*args, &block)
+            super(*args, **kwargs, &block)
           end
         end
       end
@@ -217,16 +255,7 @@ describe GraphQL::Schema do
     end
   end
 
-  describe "`use` works with plugins that attach instrumentation, tracers, query analyzers" do
-    class NoOpTracer
-      def trace(_key, data)
-        if (query = data[:query])
-          query.context[:no_op_tracer_ran] = true
-        end
-        yield
-      end
-    end
-
+  describe "`use` works with plugins that attach instrumentation, trace modules, query analyzers" do
     module NoOpTrace
       def execute_query(query:)
         query.context[:no_op_trace_ran_before_query] = true
@@ -236,7 +265,7 @@ describe GraphQL::Schema do
       end
     end
 
-    class NoOpAnalyzer < GraphQL::Analysis::AST::Analyzer
+    class NoOpAnalyzer < GraphQL::Analysis::Analyzer
       def initialize(query_or_multiplex)
         query_or_multiplex.context[:no_op_analyzer_ran_initialize] = true
         super
@@ -254,7 +283,6 @@ describe GraphQL::Schema do
     module PluginWithInstrumentationTracingAndAnalyzer
       def self.use(schema_defn)
         schema_defn.trace_with(NoOpTrace)
-        schema_defn.tracer NoOpTracer.new
         schema_defn.query_analyzer NoOpAnalyzer
       end
     end
@@ -281,7 +309,6 @@ describe GraphQL::Schema do
 
         assert_equal true, query.context[:no_op_trace_ran_before_query]
         assert_equal true, query.context[:no_op_trace_ran_after_query]
-        assert_equal true, query.context[:no_op_tracer_ran]
         assert_equal true, query.context[:no_op_analyzer_ran_initialize]
         assert_equal true, query.context[:no_op_analyzer_ran_on_leave_field]
         assert_equal true, query.context[:no_op_analyzer_ran_result]
@@ -308,7 +335,6 @@ describe GraphQL::Schema do
 
         assert_equal true, query.context[:no_op_trace_ran_before_query]
         assert_equal true, query.context[:no_op_trace_ran_after_query]
-        assert_equal true, query.context[:no_op_tracer_ran]
         assert_equal true, query.context[:no_op_analyzer_ran_initialize]
         assert_equal true, query.context[:no_op_analyzer_ran_on_leave_field]
         assert_equal true, query.context[:no_op_analyzer_ran_result]
@@ -450,4 +476,152 @@ describe GraphQL::Schema do
   it "starts with no references_to" do
     assert_equal({}, GraphQL::Schema.references_to)
   end
+
+  describe "DidYouMean support" do
+    class DidYouMeanSchema < GraphQL::Schema
+      class ExampleEnum < GraphQL::Schema::Enum
+        value "VALUE_ONE", "The first value"
+        value "VALUE_TWO", "The second value"
+      end
+
+      class Query < GraphQL::Schema::Object
+        field :first_field, String
+        field :second_field, String
+        field :second_fiel, String
+        field :scond_field, String
+        field :third_field, ExampleEnum
+      end
+
+      query(Query)
+    end
+
+    it "returns helpful messages" do
+      res = DidYouMeanSchema.execute("{ first_field }")
+      assert_equal ["Field 'first_field' doesn't exist on type 'Query' (Did you mean `firstField`?)"], res["errors"].map { |err| err["message"] }
+
+      res = DidYouMeanSchema.execute("{ seconField }")
+      assert_equal ["Field 'seconField' doesn't exist on type 'Query' (Did you mean `secondFiel`, `secondField` or `scondField`?)"], res["errors"].map { |err| err["message"] }
+    end
+
+    it "can disable those messages" do
+      no_dym_schema = Class.new(DidYouMeanSchema) do
+        did_you_mean(nil)
+      end
+      res = no_dym_schema.execute("{ first_field }")
+      assert_equal ["Field 'first_field' doesn't exist on type 'Query'"], res["errors"].map { |err| err["message"] }
+
+      res = no_dym_schema.execute("{ seconField }")
+      assert_equal ["Field 'seconField' doesn't exist on type 'Query'"], res["errors"].map { |err| err["message"] }
+    end
+
+    it "returns helpful message when non existing field is queried on a non-fields type" do
+      res = DidYouMeanSchema.execute("{ thirdField { foo } }")
+      assert_equal ["Selections can't be made on enums (field 'thirdField' returns ExampleEnum but has selections [\"foo\"])"], res["errors"].map { |err| err["message"] }
+
+      res = DidYouMeanSchema.execute("{ secondField { foo } }")
+      assert_equal ["Selections can't be made on scalars (field 'secondField' returns String but has selections [\"foo\"])"], res["errors"].map { |err| err["message"] }
+    end
+  end
+
+  it "defers root type blocks until those types are used" do
+    calls = []
+    schema = Class.new(GraphQL::Schema) do
+      use(GraphQL::Schema::Visibility)
+      query { calls << :query; Class.new(GraphQL::Schema::Object) { graphql_name("Query") } }
+      mutation { calls << :mutation; Class.new(GraphQL::Schema::Object) { graphql_name("Mutation") } }
+      subscription { calls << :subscription; Class.new(GraphQL::Schema::Object) { graphql_name("Subscription") } }
+      # Test this because it tries to modify `subscription` -- currently hardcoded in Schema.add_subscription_extension_if_necessary
+      use GraphQL::Subscriptions
+    end
+
+    assert_equal [], calls
+    assert_equal "Query", schema.query.graphql_name
+    assert_equal [:query], calls
+    assert_equal "Mutation", schema.mutation.graphql_name
+    assert_equal [:query, :mutation], calls
+    assert_equal "Subscription", schema.subscription.graphql_name
+    assert_equal [:query, :mutation, :subscription], calls
+    assert schema.instance_variable_get(:@subscription_extension_added)
+  end
+
+  it "adds the subscription extension if subscription(...) is called second" do
+    schema = Class.new(GraphQL::Schema) do
+      use GraphQL::Subscriptions
+      subscription(Class.new(GraphQL::Schema::Object) { graphql_name("Subscription") })
+    end
+    assert schema.subscription
+    assert schema.instance_variable_get(:@subscription_extension_added)
+
+    schema2 = Class.new(GraphQL::Schema) do
+      use(GraphQL::Schema::Visibility)
+      use GraphQL::Subscriptions
+      subscription(Class.new(GraphQL::Schema::Object) { graphql_name("Subscription") })
+    end
+    assert schema2.subscription
+    assert schema2.instance_variable_get(:@subscription_extension_added)
+  end
+
+  describe "backtrace error handling" do
+    class CustomError < RuntimeError; end
+    class Query < GraphQL::Schema::Object
+      field :test, Integer, null: false
+
+      def test
+        raise CustomError
+      end
+    end
+
+    it "raises a TracedError when backtrace is enabled" do
+      schema = Class.new(GraphQL::Schema) do
+        query(Query)
+        use GraphQL::Backtrace
+      end
+      query_str = '{ test }'
+
+      assert_raises(GraphQL::Backtrace::TracedError) do
+        schema.execute(query_str)
+      end
+    end
+
+    it "rescues them when using rescue_from with backtrace" do
+      schema = Class.new(GraphQL::Schema) do
+        query(Query)
+        use GraphQL::Backtrace
+
+        rescue_from(CustomError) do
+          raise GraphQL::ExecutionError.new('Handled CustomError')
+        end
+      end
+      query_str = '{ test }'
+      expected_errors = [
+        {
+          'message' => 'Handled CustomError',
+          'locations' => [{'line' => 1, 'column' => 3}],
+          'path' => ['test']
+        }
+      ]
+
+      assert_equal expected_errors, schema.execute(query_str).to_h['errors']
+    end
+  end
+  describe ".validate_timeout" do
+    it "provides a default timeout when not explicitly set" do
+      schema = Class.new(GraphQL::Schema)
+      assert_equal 3, schema.validate_timeout
+    end
+
+    it "allows overriding the default timeout" do
+      schema = Class.new(GraphQL::Schema) do
+        validate_timeout 15
+      end
+      assert_equal 15, schema.validate_timeout
+    end
+
+    it "allows disabling the timeout" do
+      schema = Class.new(GraphQL::Schema) do
+        validate_timeout nil
+      end
+      assert_nil schema.validate_timeout
+    end
+  end
 end
