@@ -49,7 +49,7 @@ describe GraphQL::StaticValidation::FieldsWillMerge do
 
       interface Pet {
         name(surname: Boolean = false): String!
-        nickname: String
+        nickname: String!
         toys: [Toy!]!
       }
 
@@ -63,7 +63,7 @@ describe GraphQL::StaticValidation::FieldsWillMerge do
 
       type Dog implements Pet & Mammal & Canine {
         name(surname: Boolean = false): String!
-        nickname: String
+        nickname: String!
         doesKnowCommand(dogCommand: PetCommand): Boolean!
         barkVolume: Int!
         toys: [Toy!]!
@@ -71,7 +71,7 @@ describe GraphQL::StaticValidation::FieldsWillMerge do
 
       type Cat implements Pet & Mammal & Feline {
         name(surname: Boolean = false): String!
-        nickname: String
+        nickname: String!
         doesKnowCommand(catCommand: PetCommand): Boolean!
         meowVolume: Int!
         toys: [Toy!]!
@@ -998,4 +998,173 @@ describe GraphQL::StaticValidation::FieldsWillMerge do
       refute_match %r/\$arg12/, error_messages.first
     end
   end
+
+  describe "Conflicting leaf typed fields" do
+    let(:schema) { GraphQL::Schema.from_definition(<<-GRAPHQL)
+      interface Thing {
+        name: String
+      }
+
+      type Dog implements Thing {
+        spots: Boolean
+      }
+
+      type Jaguar implements Thing {
+        spots: Int
+      }
+
+      type Query {
+        thing: Thing
+      }
+      GRAPHQL
+    }
+
+    let(:query_str) { <<-GRAPHQL
+        {
+          thing {
+            ... on Dog { spots }
+            ... on Jaguar { spots }
+          }
+        }
+      GRAPHQL
+    }
+
+    it "warns by default" do
+      res = nil
+      stdout, _stderr = capture_io do
+        res = schema.validate(query_str)
+      end
+      assert_equal [], res
+      expected_warning = [
+        "GraphQL-Ruby encountered mismatched types in this query: `Boolean` (at 3:26) vs. `Int` (at 4:29).",
+        "This will return an error in future GraphQL-Ruby versions, as per the GraphQL specification",
+        "Learn about migrating here: https://graphql-ruby.org/api-doc/#{GraphQL::VERSION}/GraphQL/Schema.html#allow_legacy_invalid_return_type_conflicts-class_method"
+    ].join("\n")
+      assert_includes stdout, expected_warning
+    end
+
+    it "calls the handler when legacy is enabled" do
+      legacy_schema = Class.new(schema) do
+        allow_legacy_invalid_return_type_conflicts(true)
+        def self.legacy_invalid_return_type_conflicts(query, t1, t2, node1, node2)
+
+          raise "#{query.class} / #{t1.to_type_signature} / #{t2.to_type_signature} / #{node1.position} / #{node2.position}"
+        end
+      end
+
+      err = assert_raises do
+        legacy_schema.validate(query_str)
+      end
+
+      assert_equal "GraphQL::Query / Boolean / Int / [3, 26] / [4, 29]", err.message
+    end
+
+    it "adds an error when legacy is disabled" do
+      future_schema = Class.new(schema) { allow_legacy_invalid_return_type_conflicts(false) }
+      res = future_schema.validate(query_str)
+      expected_error = {
+        "message"=>"Field 'spots' has a return_type conflict: `Boolean` or `Int`?",
+        "locations"=>[{"line"=>3, "column"=>26}, {"line"=>4, "column"=>29}],
+        "path"=>[],
+        "extensions"=>
+          {"code"=>"fieldConflict",
+           "fieldName"=>"spots",
+           "conflicts"=>"`Boolean` or `Int`"}
+        }
+      assert_equal [expected_error], res.map(&:to_h)
+    end
+
+    it "inherits allow_legacy_invalid_empty_selections_on_union" do
+      base_schema = Class.new(schema) { allow_legacy_invalid_return_type_conflicts(true) }
+      ext_schema = Class.new(base_schema)
+      assert ext_schema.allow_legacy_invalid_return_type_conflicts
+    end
+  end
+
+  describe "conflicting list / non-list fields" do
+    it "requires matching list/non-list structure" do
+      schema = GraphQL::Schema.from_definition <<~GRAPHQL
+        type Query {
+          u: U
+        }
+
+        union U = A | B
+
+        type A {
+          f: [Int]
+        }
+
+        type B {
+          f: Int
+        }
+      GRAPHQL
+
+      schema.allow_legacy_invalid_return_type_conflicts(false)
+
+      query_str = <<~GRAPHQL
+        {
+          u {
+            ... on A { f }
+            ... on B { f }
+          }
+        }
+      GRAPHQL
+
+      res = schema.validate(query_str)
+      assert_equal ["Field 'f' has a return_type conflict: `[Int]` or `Int`?"], res.map(&:message)
+
+      query_str = <<~GRAPHQL
+        {
+          u {
+            ... on B { f }
+            ... on A { f }
+          }
+        }
+      GRAPHQL
+
+      res = schema.validate(query_str)
+      assert_equal ["Field 'f' has a return_type conflict: `Int` or `[Int]`?"], res.map(&:message)
+    end
+  end
+
+  describe "duplicate aliases on a interface with inline fragment spread" do
+    class DuplicateAliasesSchema < GraphQL::Schema
+      module Node
+        include GraphQL::Schema::Interface
+        field :id, ID
+
+        def self.resolve_type(obj, ctx)
+          Repository
+        end
+      end
+
+      class Repository < GraphQL::Schema::Object
+        implements Node
+        field :name, String
+        field :id, ID
+      end
+
+      class Query < GraphQL::Schema::Object
+        field :node, Node, fallback_value: { name: "graphql-ruby", id: "abcdef" }
+      end
+
+      query(Query)
+      orphan_types(Repository)
+    end
+
+    it "returns an error" do
+      query_str = 'query {
+        node {
+          ... on Repository {
+            info: name
+            info: id
+          }
+        }
+      }
+      '
+
+      res = DuplicateAliasesSchema.execute(query_str)
+      assert_equal ["Field 'info' has a field conflict: name or id?"], res["errors"].map { |e| e["message"] }
+    end
+  end
 end
