@@ -356,15 +356,15 @@ describe GraphQL::StaticValidation::VariableUsagesAreAllowed do
 
       result = schema.execute("query($sort: SongSort) { songs(sort: $sort) { name } }", variables: {})
       expected_result = {"data" => {"songs" => [{"name" => "A"},{"name" => "B"}]}}
-      assert_equal expected_result, result
+      assert_graphql_equal expected_result, result
 
       result = schema.execute("query($sort: SongSort) { songs(sort: $sort) { name } }", variables: { sort: { name: "desc" } })
       expected_result = {"data" => {"songs" => [{"name" => "B"},{"name" => "A"}]}}
-      assert_equal expected_result, result
+      assert_graphql_equal expected_result, result
 
       result = schema.execute("query($sort: SongSort) { songs(sort: $sort) { name } }", variables: { sort: nil })
-      expected_result = {"data"=>nil, "errors"=>[{"message"=>"`null` is not a valid input for `SongSort!`, please provide a value for this argument.", "locations"=>[{"line"=>1, "column"=>26}], "path"=>["songs"]}]}
-      assert_equal expected_result, result
+      expected_result = {"errors"=>[{"message"=>"`null` is not a valid input for `SongSort!`, please provide a value for this argument.", "locations"=>[{"line"=>1, "column"=>26}], "path"=>["songs"]}], "data" => nil}
+      assert_graphql_equal expected_result, result
 
       result = schema.execute("{ topSong(input: {}) { name } }")
       assert_equal "Hey Ya!", result["data"]["topSong"]["name"]
