@@ -1,10 +1,26 @@
 # frozen_string_literal: true
-
 require 'rubygems'
 require 'bundler'
+require 'simplecov'
+require 'simplecov-lcov'
+SimpleCov::Formatter::LcovFormatter.config.report_with_single_file = true
+SimpleCov.formatters = [
+  SimpleCov::Formatter::HTMLFormatter,
+  SimpleCov::Formatter::LcovFormatter
+]
+SimpleCov.start do
+  enable_coverage :branch
+  add_filter "spec/"
+  add_group "Generators", "lib/generators"
+  add_group "Execution", [/analysis/, /language/, /execution/, /static_validation/, /pagination/, /subscriptions/, /query/, /tracing/, /introspection/, /backtrace/]
+  add_group "Helpers", [/rake_task/, /testing/, /rubocop/]
+  add_group "Definition", [/types/, /relay/, /schema/]
+  add_group "Dataloader", [/dataloader/]
+end
+
 Bundler.require
 
-# Print full backtrace for failiures:
+# Print full backtrace for failures:
 ENV["BACKTRACE"] = "1"
 
 require "graphql"
@@ -16,9 +32,21 @@ else
   USING_C_PARSER = false
 end
 
+if ENV["GRAPHQL_REJECT_NUMBERS_FOLLOWED_BY_NAMES"]
+  puts "Opting into GraphQL.reject_numbers_followed_by_names"
+  GraphQL.reject_numbers_followed_by_names = true
+  puts "Opting into GraphQL::Schema::Visibility::Profile"
+  GraphQL::Schema.use(GraphQL::Schema::Visibility, migration_errors: true)
+  ADD_WARDEN = false
+else
+  ADD_WARDEN = true
+end
+
+# C methods aren't fair game in non-main Ractors
+RUN_RACTOR_TESTS = defined?(::Ractor) && !USING_C_PARSER
+
 require "rake"
 require "graphql/rake_task"
-require "benchmark"
 require "pry"
 require "minitest/autorun"
 require "minitest/focus"
@@ -99,27 +127,22 @@ def testing_mongoid?
   defined?(::Mongoid)
 end
 
-# Load support files
-Dir["#{File.dirname(__FILE__)}/support/**/*.rb"].each do |f|
-  require f
+def testing_redis?
+  defined?(::Redis)
 end
 
 if testing_rails?
   require "integration/rails/spec_helper"
 end
 
-# Load dependencies
-['Mongoid', 'Rails'].each do |integration|
-  integration_loaded = begin
-    Object.const_get(integration)
-  rescue NameError
-    nil
-  end
-  if ENV["TEST"].nil? && integration_loaded
-    Dir["spec/integration/#{integration.downcase}/**/*.rb"].each do |f|
-      require f.sub("spec/", "")
-    end
-  end
+if testing_mongoid?
+  require "integration/mongoid/star_trek/data"
+  require "integration/mongoid/star_trek/schema"
+end
+
+# Load support files
+Dir["#{File.dirname(__FILE__)}/support/**/*.rb"].each do |f|
+  require f
 end
 
 def star_trek_query(string, variables={}, context: {})
@@ -156,7 +179,6 @@ module TestTracing
 
     def trace(key, data)
       data[:key] = key
-      data[:path] ||= data.key?(:context) ? data[:context].path : nil
       result = yield
       data[:result] = result
       traces << data
@@ -169,3 +191,39 @@ end
 if !USING_C_PARSER && defined?(GraphQL::CParser::Parser)
   raise "Load error: didn't opt in to C parser but GraphQL::CParser::Parser was defined"
 end
+
+def assert_warns(warning, printing = "")
+  return_val = nil
+  stdout, stderr = capture_io { return_val = yield }
+  assert_equal warning, stderr, "It produced the expected stderr"
+  assert_equal stdout, printing, "It produced the expected stdout"
+  return_val
+end
+
+module Minitest
+  class Test
+    def self.it_dataloads(message, &block)
+      it(message) do
+        GraphQL::Dataloader.with_dataloading do |d|
+          self.instance_exec(d, &block)
+        end
+      end
+    end
+  end
+
+  module Assertions
+    def assert_graphql_equal(data1, data2, message = "GraphQL Result was equal")
+      case data1
+      when Hash
+        assert_equal(data1, data2, message)
+        assert_equal(data1.keys, data2.keys, "Order of keys matched (#{message})")
+      when Array
+        data1.each_with_index do |item1, idx|
+          assert_graphql_equal(item1, data2[idx], message + "[Item #{idx + 1}] ")
+        end
+      else
+        raise ArgumentError, "assert_graphql_equal doesn't support #{data1.class} yet"
+      end
+    end
+  end
+end
