@@ -16,7 +16,7 @@ module StaticValidationHelpers
       target_schema = schema
       validator = GraphQL::StaticValidation::Validator.new(schema: target_schema)
       query = GraphQL::Query.new(target_schema, query_string)
-      validator.validate(query)[:errors].map(&:to_h)
+      validator.validate(query, **args)[:errors].map(&:to_h)
     end
   end
 
@@ -27,4 +27,8 @@ module StaticValidationHelpers
   def schema
     Dummy::Schema
   end
+
+  def args
+    {}
+  end
 end
